/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* This file contains my replacement for the standard yyerror() function */

#ifndef ERROR_H
#define ERROR_H

void selerror(const char *s, ...); /* yyerror */
/*
 * The string s is a format string, similar to printf.
 * The output will be: <progname>: error in line <sellineno>: s ...
 * Please note that it will ALWAYS print an extra newline.
 */

void rcerror(const char *s, ...); /* yyerror */
/* Similar to selerror in use. */

void selwarning(const char *s, ...); /* yywarning */
/*
 * The string s is a format string, similar to printf.
 * The output will be: <progname>: warning in line <sellineno>: s ...
 * Please note that it will ALWAYS print an extra newline.
 */

#ifdef OUTPUT_GUI
void selinfo(const char *s, ...); /* yyinfo */
/*
 * The string s is a format string, similar to printf.
 * The output will be: s ...
 * Please note that it will ALWAYS print an extra newline.
 */

void gui_error_init(void);
/* Initialize error window for GUI version */
#endif

#endif /* !ERROR_H */
