/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>

#ifndef CHECK_UNUSED
# ifdef DEBUG
#  define unused(x)	(x) = (x)
# else
#  define unused(x)
# endif
#else
# define unused(x)
#endif
#ifdef HIDE_UNUSED
# undef unused
# define unused(x)	(x) = (x)
#endif

#include "gtkyesno.h"

static int yesno_called;

typedef enum {none, yes, no} answer_type;
static answer_type answer;

static void yesno_sel(GtkWidget *w, answer_type current_answer)
{
	unused(w);

	answer = current_answer;
}

int gtk_yesno(char *wmclass_name, char *wmclass_class, char *title, GdkPixmap *icon,
              char *message)
{
	GtkWidget *yesno_window, *vbox;
	GtkWidget *sep, *text, *buttonbox;
	GtkWidget *yes_button, *no_button;

	if (yesno_called) {
		return 0;
	}

	yesno_called = 1;

	answer = none;

	yesno_window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_wmclass(GTK_WINDOW(yesno_window), wmclass_name, wmclass_class);
	gtk_window_set_title(GTK_WINDOW(yesno_window), title);
	gtk_widget_realize(yesno_window);
	gdk_window_set_icon(yesno_window->window, NULL, icon, NULL);
	gtk_window_set_position(GTK_WINDOW(yesno_window), GTK_WIN_POS_CENTER);
	gtk_signal_connect(GTK_OBJECT(yesno_window), "delete_event",
	                   GTK_SIGNAL_FUNC(gtk_widget_destroy), yesno_window);
	gtk_quit_add_destroy(1, GTK_OBJECT(yesno_window));
	gtk_container_set_border_width(GTK_CONTAINER(yesno_window), 10);

	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_add(GTK_CONTAINER(yesno_window), vbox);
	gtk_widget_show(vbox);

	text = gtk_label_new(message);
	gtk_container_add(GTK_CONTAINER(vbox), text);
	gtk_widget_show(text);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 1);
	gtk_widget_show(sep);

	buttonbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(buttonbox), 5);
	gtk_box_pack_end(GTK_BOX(vbox), buttonbox, FALSE, FALSE, 0);
	gtk_widget_show(buttonbox);

	yes_button = gtk_button_new_with_label("Yes");
	GTK_WIDGET_SET_FLAGS(yes_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT(yes_button), "clicked", GTK_SIGNAL_FUNC(yesno_sel),
	                   (gpointer) yes);
	gtk_box_pack_end(GTK_BOX(buttonbox), yes_button, FALSE, FALSE, 3);
	gtk_widget_grab_default(yes_button);
	gtk_widget_show(yes_button);

	no_button = gtk_button_new_with_label("No");
	GTK_WIDGET_SET_FLAGS(no_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT(no_button), "clicked", GTK_SIGNAL_FUNC(yesno_sel),
	                   (gpointer) no);
	gtk_box_pack_end(GTK_BOX(buttonbox), no_button, FALSE, FALSE, 3);
	gtk_widget_show(no_button);

	gtk_widget_show(yesno_window);

	while (answer == none) {
		gtk_main_iteration();
	}

	gtk_widget_destroy(yesno_window);
	yesno_called = 0;

	return (answer == yes);
}
