/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <limits.h>

#include <gtk/gtk.h>

#include "error.h"
#include "gdImageXpm.h"
#include "gtknumberrange.h"
#include "gtkyesno.h"
#include "gui.h"
#include "guicallbacks.h"
#include "guishow.h"
#include "guistatus.h"
#include "input.h"
#include "license.h"
#include "memalloc.h"
#include "output_epsgraph.h"
#include "output_graph.h"
#include "sel.h"
#include "strformat.h"
#include "strsave.h"
#include "tasks.h"

static unsigned long new_zoommax;
static filetype new_savetype;
static int new_transparent;

static unsigned long oldmintimemax, oldmaxtimemin;

static int load_called, save_called, opt_called, about_called;

static void file_load_ok_sel(GtkWidget *w, GtkFileSelection *fs);
static void file_load_cancel_sel(GtkWidget *w, GtkFileSelection *fs);
static void file_load_delete(GtkFileSelection *fs, GdkEvent *event, gpointer data);
static void file_save_ok_sel(GtkWidget *w, GtkFileSelection *fs);
static void file_save_cancel_sel(GtkWidget *w, GtkFileSelection *fs);
static void file_save_delete(GtkFileSelection *fs, GdkEvent *event, gpointer data);
static void options_savetype_sel(GtkWidget *w, gpointer st);
static void options_transparent_sel(GtkWidget *w, gpointer t);
static void options_ok_sel(GtkWidget *w, GtkWidget *opt_window);
static void options_cancel_sel(GtkWidget *w, GtkWidget *opt_window);
static void options_delete(GtkWidget *opt_window, GdkEvent *event, gpointer data);
static void about_ok_sel(GtkWidget *w, GtkWidget *about_window);
static void about_delete(GtkWidget *about_window, GdkEvent *event, gpointer data);
static void zoommax_change_callback(GtkNumberrange *n, unsigned long new_value);

void load_file(void)
{
	unsigned long maxtimemin, mintimemax;

	push_msg("Parsing...");
	free_tasks();
	free_resources();
	parse();

	if (number_of_tasks > 0) {
		timemin = (get_lowest_action_time_from_tasks() / 50) * 50;
		timemax = ((get_highest_action_time_from_tasks() + 49) / 50) * 50;
		if (timemax > MAXTIMEMAX) {
			timemax = MAXTIMEMAX;
		}
		if (timemax < timemin + 50) {
			timemax = timemin + 50;
		}
		if (fixedinterval) {
			oldmaxtimemin = timemin + 50;
			oldmintimemax = timemax - 50;
			if (intervalsize > timemax - timemin) {
				intervalsize = timemax - timemin;
			}
			maxtimemin = timemin + intervalsize;
			mintimemax = timemax - intervalsize;
		} else {
			maxtimemin = timemin + 50;
			mintimemax = timemax - 50;
		}
		gtk_numberrange_minimum_set(GTK_NUMBERRANGE(guimintime), timemin);
		gtk_numberrange_maximum_set(GTK_NUMBERRANGE(guimintime), mintimemax);
		gtk_numberrange_minimum_set(GTK_NUMBERRANGE(guimaxtime), maxtimemin);
		gtk_numberrange_maximum_set(GTK_NUMBERRANGE(guimaxtime), timemax);
		if (mintime < timemin) {
			mintime = timemin;
			gtk_numberrange_set(GTK_NUMBERRANGE(guimintime), mintime);
			if (fixedinterval) {
				maxtime = mintime + intervalsize;
				gtk_numberrange_set(GTK_NUMBERRANGE(guimaxtime), maxtime);
			}
		} else if (mintime > mintimemax) {
			mintime = mintimemax;
			gtk_numberrange_set(GTK_NUMBERRANGE(guimintime), mintime);
			if (fixedinterval) {
				maxtime = mintime + intervalsize;
				gtk_numberrange_set(GTK_NUMBERRANGE(guimaxtime), maxtime);
			}
		}
		if (maxtime > timemax) {
			maxtime = timemax;
			gtk_numberrange_set(GTK_NUMBERRANGE(guimaxtime), maxtime);
			if (fixedinterval) {
				mintime = maxtime - intervalsize;
				gtk_numberrange_set(GTK_NUMBERRANGE(guimintime), mintime);
			}
		} else if (maxtime < maxtimemin) {
			maxtime = maxtimemin;
			gtk_numberrange_set(GTK_NUMBERRANGE(guimaxtime), maxtime);
			if (fixedinterval) {
				mintime = maxtime - intervalsize;
				gtk_numberrange_set(GTK_NUMBERRANGE(guimintime), mintime);
			}
		}


		show();
	}
	pop_msg();
}

static void file_load_ok_sel(GtkWidget *w, GtkFileSelection *fs)
{
	char *filename;
	FILE *tmp;

	unused(w);

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	if ((filename[strlen(filename) - 1] == '/') ||
	    ((tmp = fopen(filename, "r")) == NULL ||
	     ferror(tmp))) {
		gtk_widget_destroy(GTK_WIDGET(fs));
	} else {
		fclose(tmp);
		memfree(input_filename);
		input_filename = strsave(filename);
		gtk_widget_destroy(GTK_WIDGET(fs));
		load_file();
	}
	load_called = 0;
}

static void file_load_cancel_sel(GtkWidget *w, GtkFileSelection *fs)
{
	unused(w);

	gtk_widget_destroy(GTK_WIDGET(fs));
	load_called = 0;
}

static void file_load_delete(GtkFileSelection *fs, GdkEvent *event, gpointer data)
{
	unused(event);
	unused(data);

	file_load_cancel_sel(NULL, fs);
}

void file_load_cmd_callback(GtkWidget *widget, gpointer data)
{
	GtkWidget *fs;

	unused(widget);
	unused(data);

	if (load_called) {
		return;
	}

#ifdef DEBUG
	fprintf(stderr, "File/load command called\n");
#endif

	load_called = 1;

	fs = gtk_file_selection_new("selview: load schedule");
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fs));

	GTK_WINDOW(fs)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

	gtk_widget_realize(fs);
	gdk_window_set_icon(fs->window, NULL, icon, NULL);

	/* Connect the ok_button to file_load_ok_sel function */
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
	                   GTK_SIGNAL_FUNC(file_load_ok_sel), fs);

	/* Connect the cancel_button to file_load_cancel_sel function */
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button), "clicked",
	                   GTK_SIGNAL_FUNC(file_load_cancel_sel), fs);

	/* Connect the delete event to file_load_delete function */
	gtk_signal_connect(GTK_OBJECT(fs), "delete_event",
	                   GTK_SIGNAL_FUNC(file_load_delete), NULL);

	gtk_widget_show(fs);
}

void file_reload_cmd_callback(GtkWidget *widget, gpointer data)
{
	unused(widget);
	unused(data);

	if (!strcmp(input_filename, ".")) {
		return;
	}

	load_file();
}

static int overwrite(char *filename)
{
	char *text;
	int r;

	text = memalloc((6 + strlen(filename) + 28 + 1) * sizeof(char));
	sprintf(text, "File \"%s\"\nalready exists, overwrite?", filename);

	r = gtk_yesno("overwrite", "Selview", "selview: overwrite file?", icon, text);

	memfree(text);

	return r;
}

static void file_save_ok_sel(GtkWidget *w, GtkFileSelection *fs)
{
	char *filename;
	FILE *out;
	gdImagePtr picture;
	char *epsfile;
	int errorno;

	unused(w);

	filename = strsave(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
	if (filename[strlen(filename) - 1] != '/') {
		if ((out = fopen(filename, "r"))) {
			fclose(out);
			if (!overwrite(filename)) {
				gtk_widget_destroy(GTK_WIDGET(fs));
				save_called = 0;
				memfree(filename);
				return;
			}
		}
		gtk_widget_destroy(GTK_WIDGET(fs));
		if (!((out = fopen(filename, "w")) == NULL || ferror(out))) {
			push_msg("Saving...");
			switch (savetype) {
			case eps:
				epsfile = build_eps(input_filename);
				strformat(out, epsfile, 78);
			        memfree(eps);
				break;
			case png:
				picture = build_image(file);
				gdImagePng(picture, out);
				gdImageDestroy(picture);
				break;
			case xpm:
				picture = build_image(file);
				gdImageXpm(picture, out);
				gdImageDestroy(picture);
				break;
			}
			fclose(out);
			pop_msg();
		} else {
			errorno = errno;
			if (out) {
				fclose(out);
			}
			sellineno = 0;
			selerror("writing to \"%s\": %s", filename, strerror(errorno));
		}
		save_called = 0;
	} else {
		sellineno = 0;
		selerror("cannot write to \"%s\": is a directory", filename);
	}
	memfree(filename);
}

static void file_save_cancel_sel(GtkWidget *w, GtkFileSelection *fs)
{
	unused(w);

	gtk_widget_destroy(GTK_WIDGET(fs));
	save_called = 0;
}

static void file_save_delete(GtkFileSelection *fs, GdkEvent *event, gpointer data)
{
	unused(event);
	unused(data);

	file_save_cancel_sel(NULL, fs);
}

void file_save_cmd_callback(GtkWidget *widget, gpointer data)
{
	GtkWidget *fs;
	char *output_filename;

	unused(widget);
	unused(data);

	if (!strcmp(input_filename, ".")) {
		return;
	}

	if (save_called) {
		return;
	}

#ifdef DEBUG
	fprintf(stderr, "File/save command called\n");
#endif

	save_called = 1;

	fs = NULL;
	output_filename = strsave(input_filename);
	output_filename = realloc(output_filename, strlen(output_filename) + 5);
	if (!strcmp(&output_filename[strlen(output_filename) - 4], ".sel")) {
		output_filename[strlen(output_filename) - 4] = '\0';
	}
	switch (savetype) {
	case eps:
		fs = gtk_file_selection_new("selview: save schedule as EPS");
		strcat(output_filename, ".eps");
		break;
	case png:
		fs = gtk_file_selection_new("selview: save schedule as PNG");
		strcat(output_filename, ".png");
		break;
	case xpm:
		fs = gtk_file_selection_new("selview: save schedule as XPM");
		strcat(output_filename, ".xpm");
		break;
	}
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs), output_filename);
	memfree(output_filename);
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fs));

	GTK_WINDOW(fs)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

	gtk_widget_realize(fs);
	gdk_window_set_icon(fs->window, NULL, icon, NULL);

	/* Connect the ok_button to file_save_ok_sel function */
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->ok_button), "clicked",
	                   GTK_SIGNAL_FUNC(file_save_ok_sel), fs);

	/* Connect the cancel_button to file_save_cancel_sel function */
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button), "clicked",
	                   GTK_SIGNAL_FUNC(file_save_cancel_sel), fs);

	/* Connect the delete event to file_save_delete function */
	gtk_signal_connect(GTK_OBJECT(fs), "delete_event",
	                   GTK_SIGNAL_FUNC(file_save_delete), NULL);

	gtk_widget_show(fs);
}

static void options_savetype_sel(GtkWidget *w, gpointer st)
{
	unused(w);

	new_savetype = (filetype) st;
}

static void options_transparent_sel(GtkWidget *w, gpointer t)
{
	unused(w);

	new_transparent = (int) t;
}

static void options_ok_sel(GtkWidget *w, GtkWidget *opt_window)
{
	int redraw;

	unused(w);

	redraw = 0;
	if (zoom > new_zoommax) {
		zoom = new_zoommax;
		redraw = 1;
	}

	savetype = new_savetype;
	transparent = new_transparent;

	zoommax = new_zoommax;
	gtk_numberrange_maximum_set(GTK_NUMBERRANGE(guizoom), zoommax);
	gtk_numberrange_set(GTK_NUMBERRANGE(guizoom), zoom);

	gtk_widget_destroy(opt_window);

	if (redraw) {
		if (strcmp(input_filename, ".")) {
			if (number_of_tasks > 0) {
				show();
			}
		}
	}

	opt_called = 0;
}

static void options_cancel_sel(GtkWidget *w, GtkWidget *opt_window)
{
	unused(w);

	gtk_widget_destroy(opt_window);
	opt_called = 0;
}

static void options_delete(GtkWidget *opt_window, GdkEvent *event, gpointer data)
{
	unused(event);
	unused(data);

	options_cancel_sel(NULL, opt_window);
}

void file_options_cmd_callback(GtkWidget *widget, gpointer data)
{
	GtkWidget *opt_window, *vbox;
	GtkWidget *hbox_transparent, *hbox_savetype, *hbox_zoommax, *sep;
	GtkWidget *buttonbox;
	GtkWidget *opt_zoommax;
	GtkWidget *savetype_text, *savetype_menu, *savetype_optionmenu;
	GtkWidget *savetype_eps, *savetype_png, *savetype_xpm;
	GtkWidget *transparent_text, *transparent_menu, *transparent_optionmenu;
	GtkWidget *transparent_true, *transparent_false;
	GtkWidget *ok_button, *cancel_button;

	unused(widget);
	unused(data);

	if (opt_called) {
		return;
	}

#ifdef DEBUG
	fprintf(stderr, "File/options command called\n");
#endif

	opt_called = 1;

	new_savetype = savetype;
	new_transparent = transparent;
	new_zoommax = zoommax;

	opt_window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_wmclass(GTK_WINDOW(opt_window), "options", "Selview");
	gtk_window_set_title(GTK_WINDOW(opt_window), "selview: options");
	gtk_widget_realize(opt_window);
	gdk_window_set_icon(opt_window->window, NULL, icon, NULL);
	gtk_window_set_position(GTK_WINDOW(opt_window), GTK_WIN_POS_CENTER);
	gtk_signal_connect(GTK_OBJECT(opt_window), "delete_event",
	                   GTK_SIGNAL_FUNC(options_delete), NULL);
	gtk_quit_add_destroy(1, GTK_OBJECT(opt_window));
	gtk_container_set_border_width(GTK_CONTAINER(opt_window), 10);

	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_add(GTK_CONTAINER(opt_window), vbox);
	gtk_widget_show(vbox);

	hbox_savetype = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox_savetype, FALSE, FALSE, 1);
	gtk_widget_show(hbox_savetype);

	savetype_text = gtk_label_new("Select file type for saving ");
	gtk_container_add(GTK_CONTAINER(hbox_savetype), savetype_text);
	gtk_widget_show(savetype_text);

	savetype_menu = gtk_menu_new();
	savetype_eps = gtk_menu_item_new_with_label("EPS");
	gtk_menu_append(GTK_MENU(savetype_menu), savetype_eps);
	gtk_signal_connect(GTK_OBJECT(savetype_eps), "activate",
	                   GTK_SIGNAL_FUNC(options_savetype_sel), (gpointer) eps);
	gtk_widget_show(savetype_eps);
	savetype_png = gtk_menu_item_new_with_label("PNG");
	gtk_menu_append(GTK_MENU(savetype_menu), savetype_png);
	gtk_signal_connect(GTK_OBJECT(savetype_png), "activate",
	                   GTK_SIGNAL_FUNC(options_savetype_sel), (gpointer) png);
	gtk_widget_show(savetype_png);
	savetype_xpm = gtk_menu_item_new_with_label("XPM");
	gtk_menu_append(GTK_MENU(savetype_menu), savetype_xpm);
	gtk_signal_connect(GTK_OBJECT(savetype_xpm), "activate",
	                   GTK_SIGNAL_FUNC(options_savetype_sel), (gpointer) xpm);
	gtk_widget_show(savetype_xpm);
	savetype_optionmenu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(savetype_optionmenu), savetype_menu);
	switch (savetype) {
	case eps:
		gtk_option_menu_set_history(GTK_OPTION_MENU(savetype_optionmenu), 0);
		break;
	case png:
		gtk_option_menu_set_history(GTK_OPTION_MENU(savetype_optionmenu), 1);
		break;
	case xpm:
		gtk_option_menu_set_history(GTK_OPTION_MENU(savetype_optionmenu), 2);
		break;
	}
	gtk_container_add(GTK_CONTAINER(hbox_savetype), savetype_optionmenu);
	gtk_widget_show(savetype_optionmenu);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 1);
	gtk_widget_show(sep);

	hbox_transparent = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox_transparent, FALSE, FALSE, 1);
	gtk_widget_show(hbox_transparent);

	transparent_text = gtk_label_new("Transparent background for saving ");
	gtk_container_add(GTK_CONTAINER(hbox_transparent), transparent_text);
	gtk_widget_show(transparent_text);

	transparent_menu = gtk_menu_new();
	transparent_true = gtk_menu_item_new_with_label("Yes");
	gtk_menu_append(GTK_MENU(transparent_menu), transparent_true);
	gtk_signal_connect(GTK_OBJECT(transparent_true), "activate",
	                   GTK_SIGNAL_FUNC(options_transparent_sel), (gpointer) 1);
	gtk_widget_show(transparent_true);
	transparent_false = gtk_menu_item_new_with_label("No");
	gtk_menu_append(GTK_MENU(transparent_menu), transparent_false);
	gtk_signal_connect(GTK_OBJECT(transparent_false), "activate",
	                   GTK_SIGNAL_FUNC(options_transparent_sel), (gpointer) 0);
	gtk_widget_show(transparent_false);
	transparent_optionmenu = gtk_option_menu_new();
	gtk_option_menu_set_menu(GTK_OPTION_MENU(transparent_optionmenu), transparent_menu);
	if (transparent) {
		gtk_option_menu_set_history(GTK_OPTION_MENU(transparent_optionmenu), 0);
	} else {
		gtk_option_menu_set_history(GTK_OPTION_MENU(transparent_optionmenu), 1);
	}
	gtk_container_add(GTK_CONTAINER(hbox_transparent), transparent_optionmenu);
	gtk_widget_show(transparent_optionmenu);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 1);
	gtk_widget_show(sep);

	hbox_zoommax = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox_zoommax, FALSE, FALSE, 1);
	gtk_widget_show(hbox_zoommax);

	opt_zoommax = gtk_numberrange_new("Maximum zoom: ", zoommax, 1, 20, 1, 5,
	                                  zoommax_change_callback);
	gtk_box_pack_end(GTK_BOX(hbox_zoommax), opt_zoommax, FALSE, FALSE, 1);
	gtk_widget_show(opt_zoommax);

	sep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 1);
	gtk_widget_show(sep);

	buttonbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(buttonbox), GTK_BUTTONBOX_END);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(buttonbox), 5);
	gtk_box_pack_end(GTK_BOX(vbox), buttonbox, FALSE, FALSE, 0);
	gtk_widget_show(buttonbox);

	ok_button = gtk_button_new_with_label("OK");
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
	                   GTK_SIGNAL_FUNC(options_ok_sel), opt_window);
	gtk_box_pack_end(GTK_BOX(buttonbox), ok_button, FALSE, FALSE, 3);
	gtk_widget_grab_default(ok_button);
	gtk_widget_show(ok_button);

	cancel_button = gtk_button_new_with_label("Cancel");
	GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
	                   GTK_SIGNAL_FUNC(options_cancel_sel), opt_window);
	gtk_box_pack_end(GTK_BOX(buttonbox), cancel_button, FALSE, FALSE, 3);
	gtk_widget_show(cancel_button);

	gtk_widget_show(opt_window);
}

static void about_ok_sel(GtkWidget *w, GtkWidget *about_window)
{
	unused(w);

	gtk_widget_destroy(about_window);
	about_called = 0;
}

static void about_delete(GtkWidget *about_window, GdkEvent *event, gpointer data)
{
	unused(event);
	unused(data);

	about_ok_sel(NULL, about_window);
}

void file_about_cmd_callback(GtkWidget *widget, gpointer data)
{
	GtkWidget *about_window, *vbox, *bbox, *textbox, *textwin, *textscroll, *ok_button;

	unused(widget);
	unused(data);

	if (about_called) {
		return;
	}
	about_called = 1;

	about_window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_wmclass(GTK_WINDOW(about_window), "about", "Selview");
	gtk_window_set_title(GTK_WINDOW(about_window), "selview: about");
	gtk_widget_realize(about_window);
	gdk_window_set_icon(about_window->window, NULL, icon, NULL);
	gtk_window_set_position(GTK_WINDOW(about_window), GTK_WIN_POS_CENTER);
	gtk_signal_connect(GTK_OBJECT(about_window), "delete_event",
	                   GTK_SIGNAL_FUNC(about_delete), NULL);
	gtk_quit_add_destroy(1, GTK_OBJECT(about_window));
	gtk_container_set_border_width(GTK_CONTAINER(about_window), 10);

	vbox = gtk_vbox_new(FALSE, 10);
	gtk_container_add(GTK_CONTAINER(about_window), vbox);
	gtk_widget_show(vbox);

	textbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(vbox), textbox);
	gtk_widget_show(textbox);

	textwin = gtk_text_new(NULL, NULL);
	gtk_text_set_editable(GTK_TEXT(textwin), FALSE);
	gtk_widget_set_usize(textwin, 450, 180);
	gtk_container_add(GTK_CONTAINER(textbox), textwin);
	gtk_widget_realize(textwin);
	gtk_text_insert(GTK_TEXT(textwin), NULL, NULL, NULL, license_text, -1);
	gtk_widget_show(textwin);

	textscroll = gtk_vscrollbar_new(GTK_TEXT(textwin)->vadj);
	gtk_box_pack_start(GTK_BOX(textbox), textscroll, FALSE, FALSE, 0);
	gtk_widget_show(textscroll);

	bbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
	gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);
	gtk_widget_show(bbox);

	ok_button = gtk_button_new_with_label("OK");
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(ok_button);
	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
	                   GTK_SIGNAL_FUNC(about_ok_sel), about_window);
	gtk_container_add(GTK_CONTAINER(bbox), ok_button);
	gtk_widget_show(ok_button);

	gtk_widget_show(about_window);
}

gint file_quit_cmd_callback(GtkWidget *widget, gint type)
{
	unused(widget);

#ifdef DEBUG
	fprintf(stderr, "File/quit command called\n");
#endif

	if (askquit && strcmp(input_filename, ".")) {
		if (gtk_yesno("quit", "Selview", "selview: quit?", icon,
		              "Are you sure you want to quit?")) {
			gtk_exit(0);
		}
	} else {
		gtk_exit(0);
	}
	return type;
}

void zoom_change_callback(GtkNumberrange *n, unsigned long new_zoom)
{
	unused(n);

#ifdef DEBUG
	fprintf(stderr, "zoom_change_callback called: new_zoom = %lu\n", new_zoom);
#endif

	if ((unsigned int)new_zoom != zoom) {
		zoom = (unsigned int)new_zoom;
		if (strcmp(input_filename, ".")) {
			if (number_of_tasks > 0) {
				show();
			}
		}
	}
}

void maxtime_change_callback(GtkNumberrange *n, unsigned long new_maxtime)
{
	int mintime_changed;

	unused(n);

#ifdef DEBUG
	fprintf(stderr, "maxtime_change_callback called: new_maxtime = %lu\n", new_maxtime);
#endif

	mintime_changed = 0;
	if (new_maxtime != maxtime) {
		if (fixedinterval) {
			mintime = new_maxtime - intervalsize;
			mintime_changed = 1;
		} else {
			if (mintime + 50 > new_maxtime) {
				mintime = new_maxtime - 50;
				mintime_changed = 1;
			}
			if (zoommax * (new_maxtime - mintime) > ZOOM_TIME_PRODUCT_MAX) {
				mintime = new_maxtime - ((ZOOM_TIME_PRODUCT_MAX / zoommax + 49) / 50) * 50;
				mintime_changed = 1;
			}
		}
		maxtime = new_maxtime;
		if (mintime_changed) {
			gtk_numberrange_set(GTK_NUMBERRANGE(guimintime), mintime);
		}
		if (strcmp(input_filename, ".")) {
			if (number_of_tasks > 0) {
				show();
			}
		}
	}
}

void mintime_change_callback(GtkNumberrange *n, unsigned long new_mintime)
{
	int maxtime_changed;

	unused(n);

#ifdef DEBUG
	fprintf(stderr, "mintime_change_callback called: new_mintime = %lu\n", new_mintime);
#endif

	maxtime_changed = 0;
	if (new_mintime != mintime) {
		if (fixedinterval) {
			maxtime = new_mintime + intervalsize;
			maxtime_changed = 1;
		} else {
			if (new_mintime + 50 > maxtime) {
				maxtime = new_mintime + 50;
				maxtime_changed = 1;
			}
			if (zoommax * (maxtime - new_mintime) > ZOOM_TIME_PRODUCT_MAX) {
				maxtime = new_mintime + ((ZOOM_TIME_PRODUCT_MAX / zoommax + 49) / 50) * 50;
				maxtime_changed = 1;
			}
		}
		mintime = new_mintime;
		if (maxtime_changed) {
			gtk_numberrange_set(GTK_NUMBERRANGE(guimaxtime), maxtime);
		}
		if (strcmp(input_filename, ".")) {
			if (number_of_tasks > 0) {
				show();
			}
		}
	}
}

static void zoommax_change_callback(GtkNumberrange *n, unsigned long new_value)
{
	unused(n);

#ifdef DEBUG
	fprintf(stderr, "zoommax_change_callback called: new_value = %lu\n", new_value);
#endif

	if (new_value != new_zoommax) {
		if (new_value * (maxtime - mintime) > ZOOM_TIME_PRODUCT_MAX) {
			maxtime = mintime + ((ZOOM_TIME_PRODUCT_MAX / new_value + 49) / 50) * 50;
			gtk_numberrange_set(GTK_NUMBERRANGE(guimaxtime), maxtime);
		}
		new_zoommax = new_value;
	}
}

void fixedinterval_change_callback(GtkWidget *widget, gpointer data)
{
	unused(data);

	if (GTK_TOGGLE_BUTTON(widget)->active) {
		/* Fixed interval ON */
#ifdef DEBUG
		fprintf(stderr, "Turning fixed interval ON\n");
#endif
		fixedinterval = 1;
		intervalsize = maxtime - mintime;
		oldmintimemax = gtk_numberrange_maximum_get(GTK_NUMBERRANGE(guimintime));
		gtk_numberrange_maximum_set(GTK_NUMBERRANGE(guimintime), timemax - intervalsize);
		oldmaxtimemin = gtk_numberrange_minimum_get(GTK_NUMBERRANGE(guimaxtime));
		gtk_numberrange_minimum_set(GTK_NUMBERRANGE(guimaxtime), intervalsize);
		gtk_numberrange_big_change_set(GTK_NUMBERRANGE(guimintime), intervalsize);
		gtk_numberrange_big_change_set(GTK_NUMBERRANGE(guimaxtime), intervalsize);
		if (intervalsize == 50) {
			gtk_numberrange_change_set(GTK_NUMBERRANGE(guimintime), 10);
			gtk_numberrange_change_set(GTK_NUMBERRANGE(guimaxtime), 10);
		}
		gtk_label_set_text(GTK_LABEL(fixedinterval_label), "fixed");
	} else {
		/* Fixed interval OFF */
#ifdef DEBUG
		fprintf(stderr, "Turning fixed interval OFF\n");
#endif
		fixedinterval = 0;
		gtk_numberrange_maximum_set(GTK_NUMBERRANGE(guimintime), oldmintimemax);
		gtk_numberrange_minimum_set(GTK_NUMBERRANGE(guimaxtime), oldmaxtimemin);
		gtk_numberrange_change_set(GTK_NUMBERRANGE(guimintime), 50);
		gtk_numberrange_big_change_set(GTK_NUMBERRANGE(guimintime), 1000);
		gtk_numberrange_change_set(GTK_NUMBERRANGE(guimaxtime), 50);
		gtk_numberrange_big_change_set(GTK_NUMBERRANGE(guimaxtime), 1000);
		gtk_label_set_text(GTK_LABEL(fixedinterval_label), "variable");
	}
#ifdef DEBUG
	fprintf(stderr, "GUI: mintimemin = %lu <= mintime = %lu <= mintimemax = %lu || mintime = %lu\n",
	                gtk_numberrange_minimum_get(GTK_NUMBERRANGE(guimintime)),
	                gtk_numberrange_get(GTK_NUMBERRANGE(guimintime)),
	                gtk_numberrange_maximum_get(GTK_NUMBERRANGE(guimintime)),
	                mintime);
	fprintf(stderr, "GUI: maxtimemin = %lu <= maxtime = %lu <= maxtimemax = %lu || maxtime = %lu\n",
	                gtk_numberrange_minimum_get(GTK_NUMBERRANGE(guimaxtime)),
	                gtk_numberrange_get(GTK_NUMBERRANGE(guimaxtime)),
	                gtk_numberrange_maximum_get(GTK_NUMBERRANGE(guimaxtime)),
	                maxtime);
#endif
}
