/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <gd.h>

#include <gtk/gtk.h>

#include "gdImageXpm.h"
#include "gui.h"
#include "guistatus.h"
#include "memalloc.h"
#include "output_graph.h"
#include "sel.h"

void show(void)
{
	char **new_pixmap_data;
	GdkPixmap *new_pixmap;
	GtkStyle *style;
	gint w, h;
	char *new_title;
	static int drawing = 0;
	static int needredraw = 0;

	if (drawing) {
		needredraw = 1;
		return;
	}

#ifdef DEBUG
	fprintf(stderr, "show: start\n");
#endif

	drawing = 1;

	push_msg("Drawing image...");

	image = build_image(screen);

	push_msg("Creating pixmap...");

	style = gtk_widget_get_style(main_window);
	new_pixmap_data = gdCreateXpmDataFromImage(image);
	gdImageDestroy(image);

	push_msg("Displaying pixmap...");

	new_pixmap = gdk_pixmap_create_from_xpm_d(main_window->window, &mask,
	                                          &style->bg[GTK_STATE_NORMAL], new_pixmap_data);
	gdDestroyXpmData(new_pixmap_data);

	/* dirty trick to remove the old pixmap */
	gdk_window_get_size(pixmapwid->window, &w, &h);
	gdk_draw_rectangle(pixmapwid->window, style->bg_gc[GTK_STATE_NORMAL], 1, 0, 0, w, h);

	/* replace old pixmap with new one */
	gtk_pixmap_set(GTK_PIXMAP(pixmapwid), new_pixmap, mask);
	gdk_pixmap_unref(pixmap);
	pixmap = new_pixmap;

	/* insert input_filename into window title */
	/* Total length = length("selview: ") + length filename + null-byte */
	new_title = memalloc((9 + strlen(input_filename) + 1) * sizeof(char));
	strcpy(new_title, "selview: ");
	strcat(new_title, input_filename);
	gtk_window_set_title(GTK_WINDOW(main_window), new_title);
	gdk_window_set_icon_name(main_window->window, "selview");
	memfree(new_title);

	pop_msg();
	pop_msg();
	pop_msg();

	drawing = 0;

	if (needredraw) {
		needredraw = 0;
		show();
	}

#ifdef DEBUG
	fprintf(stderr, "show: end\n");
#endif
}
