/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "error.h"
#include "sel.h"
#include "sel.parser.h"
#include "sel.scanner.h"

void parse(void)
{
	FILE *input;

	sellineno = 0;
#ifdef OUTPUT_GUI
	if (!strcmp(input_filename, ".")) { /* should never happen within the GUI */
		selerror("cannot read from stdin");
		return;
	}
	selinfo("------ File: %s ------", input_filename);
#endif
	if (!strcmp(input_filename, ".")) { /* if filename is ".", use stdin */
		input = stdin;
	} else {
		if ((input = fopen(input_filename, "r")) == NULL || ferror(input)) {
			selerror("cannot open %s", input_filename);
#ifndef OUTPUT_GUI
			exit(1);
#else
			return;
#endif
		}
	}

	selin = input;
	sellineno = 1;
#if YYDEBUG != 0
	seldebug = 1;
#endif

#ifdef DEBUG
	fprintf(stderr, "selparse: start\n");
#endif
	if (selparse()) {
#ifndef OUTPUT_GUI
		exit(2);
#else
		selrestart(selin);
#endif
	}
#ifdef DEBUG
	fprintf(stderr, "selparse: end\n");
#endif
	sellineno = 0;

	if (input != stdin) {
		fclose(input);
	}
}
