/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>

#include "doatexit.h"
#include "error.h"
#include "input.h"
#include "options.h"
#include "output.h"
#include "sel.h"
#include "sel.parser.h"
#include "tasks.h"

#ifdef PLUMBER
# include <plumber.h>
#endif

int main(int argc, char *argv[])
{
	FILE *output;

	progname = argv[0];
#ifdef PLUMBER
	plumber_init(progname);
	if (atexit(plumber_shutdown)) {
		selerror("cannot register plumber_shutdown() with atexit()");
		plumber_shutdown();
		exit(1);
	}
#endif

	process_options(argc, argv);
	doatexit(cleanup_options);

	doatexit(free_tasks);
	doatexit(free_resources);

	parse();

	if (output_filename) {
		if ((output = fopen(output_filename, "wb")) == NULL || ferror(output)) {
			selerror("cannot open %s", output_filename);
			return 1;
		}
	} else {
		output = stdout;
	}

	show(output);

	fclose(output);

	return 0;
}
