/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Warning: this file contains very much #ifdef regions */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>

#include "error.h"
#include "get_cwd.h"
#include "getopt.h"
#ifdef OUTPUT_GUI
# include "gui.h"
#endif
#include "license.h"
#include "memalloc.h"
#include "rc.parser.h"
#include "rc.scanner.h"
#include "sel.h"
#include "strsave.h"
#include "version.h"

void usage(void)
{
	fprintf(stderr, "Usage: %s [options] [input-file-name]\n" ,progname);
	fprintf(stderr, "Options:\n");
#ifndef OUTPUT_GUI
	fprintf(stderr, " -o, --output output-file-name\n");
#endif
#ifdef OUTPUT_GRAPH
	fprintf(stderr, " -L, --lower-limit min-time\n");
#endif
	fprintf(stderr, " -H, --upper-limit max-time\n");
#ifdef OUTPUT_NUM
	fprintf(stderr, " -v, --verbose\n");
	fprintf(stderr, " -d, --data\n");
#endif
#ifdef OUTPUT_GRAPH
	fprintf(stderr, " -z, --zoom zoom-factor\n");
	fprintf(stderr, " -t, --transparent\n");
	fprintf(stderr, " -T, --non-transparent (default)\n");
#endif
#ifdef OUTPUT_GUI
	fprintf(stderr, " -Z, --zoom-limit max-zoom-factor\n");
	fprintf(stderr, " -s, --savetype eps|png|xpm\n");
	fprintf(stderr, " -f, --fixed-interval\n");
	fprintf(stderr, " -F, --no-fixed-interval (default)\n");
	fprintf(stderr, " -q, --ask-quit (default)\n");
	fprintf(stderr, " -Q, --no-ask-quit\n");
#endif
	fprintf(stderr, " -r, --rcfile rcfile-file-name\n");
	fprintf(stderr, "     --no-rcfile\n");
	fprintf(stderr, " -c, --copyright\n");
	fprintf(stderr, " -V, --version\n");
	fprintf(stderr, " -h, --help\n");
#ifdef OUTPUT_GUI
	fprintf(stderr, " *: and the options belonging to the gtk library\n");
#endif
	exit(1);
}

void version(void)
{
	printf("%s v" VERSION " by " AUTHOR "\n", progname);
	exit(0);
}

void copyright(void)
{
	printf(license_text);
	exit(0);
}

void process_options(unsigned int argc, char *argv[])
{
	/* Initialize structures with possible options */
#ifdef OUTPUT_GRAPH
# ifndef OUTPUT_GUI
	char short_opts[] = "o:L:H:z:tTr:cVh";
# else
	char short_opts[] = "L:H:z:tTZ:s:fFqQr:cVh";
# endif
#else
	char short_opts[] = "o:H:vdr:cVh";
#endif
	struct option long_opts[] = {
#ifndef OUTPUT_GUI
		{"output"           , required_argument, NULL, 'o'},
#endif
#ifdef OUTPUT_GRAPH
		{"lowerlimit"       , required_argument, NULL, 'L'},
		{"lower-limit"      , required_argument, NULL, 'L'},
		{"lower_limit"      , required_argument, NULL, 'L'},
#endif
		{"upperlimit"       , required_argument, NULL, 'H'},
		{"upper-limit"      , required_argument, NULL, 'H'},
		{"upper_limit"      , required_argument, NULL, 'H'},
#ifdef OUTPUT_NUM
		{"verbose"          , no_argument      , NULL, 'v'},
		{"data"             , no_argument      , NULL, 'd'},
#endif
#ifdef OUTPUT_GRAPH
		{"zoom"             , required_argument, NULL, 'z'},
		{"transparent"      , no_argument      , NULL, 't'},
		{"nontransparent"   , no_argument      , NULL, 'T'},
		{"non-transparent"  , no_argument      , NULL, 'T'},
		{"non_transparent"  , no_argument      , NULL, 'T'},
#endif
#ifdef OUTPUT_GUI
		{"zoomlimit"        , required_argument, NULL, 'Z'},
		{"zoom-limit"       , required_argument, NULL, 'Z'},
		{"zoom_limit"       , required_argument, NULL, 'Z'},
		{"savetype"         , required_argument, NULL, 's'},
		{"save-type"        , required_argument, NULL, 's'},
		{"save_type"        , required_argument, NULL, 's'},
		{"fixedinterval"    , no_argument,       NULL, 'f'},
		{"fixed-interval"   , no_argument,       NULL, 'f'},
		{"fixed_interval"   , no_argument,       NULL, 'f'},
		{"nofixedinterval"  , no_argument,       NULL, 'F'},
		{"nofixed-interval" , no_argument,       NULL, 'F'},
		{"nofixed_interval" , no_argument,       NULL, 'F'},
		{"no-fixedinterval" , no_argument,       NULL, 'F'},
		{"no-fixed-interval", no_argument,       NULL, 'F'},
		{"no-fixed_interval", no_argument,       NULL, 'F'},
		{"no_fixedinterval" , no_argument,       NULL, 'F'},
		{"no_fixed-interval", no_argument,       NULL, 'F'},
		{"no_fixed_interval", no_argument,       NULL, 'F'},
		{"askquit"          , no_argument,       NULL, 'q'},
		{"ask-quit"         , no_argument,       NULL, 'q'},
		{"ask_quit"         , no_argument,       NULL, 'q'},
		{"noaskquit"        , no_argument,       NULL, 'Q'},
		{"noask-quit"       , no_argument,       NULL, 'Q'},
		{"noask_quit"       , no_argument,       NULL, 'Q'},
		{"no-askquit"       , no_argument,       NULL, 'Q'},
		{"no-ask-quit"      , no_argument,       NULL, 'Q'},
		{"no-ask_quit"      , no_argument,       NULL, 'Q'},
		{"no_askquit"       , no_argument,       NULL, 'Q'},
		{"no_ask-quit"      , no_argument,       NULL, 'Q'},
		{"no_ask_quit"      , no_argument,       NULL, 'Q'},
#endif
		{"rcfile"           , required_argument, NULL, 'r'},
		{"rc-file"          , required_argument, NULL, 'r'},
		{"rc_file"          , required_argument, NULL, 'r'},
		{"norcfile"         , no_argument      , NULL, 'R'},
		{"norc-file"        , no_argument      , NULL, 'R'},
		{"norc_file"        , no_argument      , NULL, 'R'},
		{"no-rcfile"        , no_argument      , NULL, 'R'},
		{"no-rc-file"       , no_argument      , NULL, 'R'},
		{"no-rc_file"       , no_argument      , NULL, 'R'},
		{"no_rcfile"        , no_argument      , NULL, 'R'},
		{"no_rc-file"       , no_argument      , NULL, 'R'},
		{"no_rc_file"       , no_argument      , NULL, 'R'},
		{"copyright"        , no_argument      , NULL, 'c'},
		{"version"          , no_argument      , NULL, 'V'},
		{"help"             , no_argument      , NULL, 'h'},
		{0                  , 0                , NULL, 0}
	};

	/* Temporary variables */
	int c;
#ifdef OUTPUT_GUI
	char *st;
	enum {o_none, o_L, o_H, o_z, o_Z} last_option = o_none;
#endif

	/* opt_* variables with default values */
	char *opt_rcfile_filename = NULL; /* Meaning ~/.sel_rc */
#ifndef OUTPUT_GUI
	char *opt_output_filename = NULL;
#endif
#ifdef OUTPUT_NUM
	unsigned long opt_verbosity = 0;
	unsigned long opt_data = 0;
	int opt_data_verbosity_b = 0;
#endif
#ifdef OUTPUT_GRAPH
	unsigned long opt_zoom = 1;
	int opt_zoom_b = 0;
	unsigned long opt_transparent = 0;
	int opt_transparent_b = 0;
#endif
#ifdef OUTPUT_GUI
	unsigned long opt_zoommax = 8;
	int opt_zoommax_b = 0;
	filetype opt_savetype = png;
	int opt_savetype_b = 0;
	int opt_fixedinterval = 0;
	int opt_fixedinterval_b = 0;
	int opt_askquit = 1;
	int opt_askquit_b = 0;
	unsigned long opt_maxtime = 4000;
#else
	unsigned long opt_maxtime = ((ULONG_MAX - 50) / 1000) * 1000;
#endif
	int opt_maxtime_b = 0;
#ifdef OUTPUT_GRAPH
	unsigned long opt_mintime = 0;
	int opt_mintime_b = 0;
#endif
	int opt_no_rcfile = 0;

	/* Assign defaults (which are in opt_* already) to global option variables */
#ifdef OUTPUT_GRAPH
	mintime = opt_mintime;
#endif
	maxtime = opt_maxtime;
#ifdef OUTPUT_NUM
	verbosity = opt_verbosity;
	data = opt_data;
#endif
#ifdef OUTPUT_GRAPH
	zoom = opt_zoom;
	transparent = opt_transparent;
#endif
#ifdef OUTPUT_GUI
	zoommax = opt_zoommax;
	savetype = opt_savetype;
	fixedinterval = opt_fixedinterval;
	askquit = opt_askquit;
#endif

	/* Process options and place results in opt_* variables */
	while ((c = getopt_long((int) argc, argv, short_opts, long_opts, NULL)) != EOF) {
		switch (c) {
#ifndef OUTPUT_GUI
		case 'o':
			opt_output_filename = strsave(optarg);
			break;
#endif
#ifdef OUTPUT_GRAPH
		case 'L':
			opt_mintime = strtoul(optarg, NULL, 10);
			opt_mintime_b = 1;
# ifdef OUTPUT_GUI
			last_option = o_L;
# endif
			break;
#endif
		case 'H':
			opt_maxtime = strtoul(optarg, NULL, 10);
			opt_maxtime_b = 1;
#ifdef OUTPUT_GUI
			last_option = o_H;
#endif
			break;
#ifdef OUTPUT_NUM
		case 'v':
			opt_verbosity = 1;
			opt_data = 0;
			opt_data_verbosity_b = 1;
			break;
		case 'd':
			opt_data = 1;
			opt_verbosity = 0;
			opt_data_verbosity_b = 1;
			break;
#endif
#ifdef OUTPUT_GRAPH
		case 'z':
			opt_zoom = strtoul(optarg, NULL, 10);
			opt_zoom_b = 1;
#ifdef OUTPUT_GUI
			last_option = o_z;
#endif
			break;
		case 't':
			opt_transparent = 1;
			opt_transparent_b = 1;
			break;
		case 'T':
			opt_transparent = 0;
			opt_transparent_b = 1;
			break;
#endif
#ifdef OUTPUT_GUI
		case 'Z':
			opt_zoommax = strtoul(optarg, NULL, 10);
			opt_zoommax_b = 1;
#ifdef OUTPUT_GUI
			last_option = o_Z;
#endif
			break;
		case 's':
			st = strsave(optarg);
			if (!strncasecmp(st, "eps", 3)) {
				opt_savetype = eps;
			} else if (!strncasecmp(st, "png", 3)) {
				opt_savetype = png;
			} else if (!strncasecmp(st, "xpm", 3)) {
				opt_savetype = xpm;
			} else {
				usage(); /* does not return */
			}
			memfree(st);
			opt_savetype_b = 1;
			break;
		case 'f':
			opt_fixedinterval = 1;
			opt_fixedinterval_b = 1;
			break;
		case 'F':
			opt_fixedinterval = 0;
			opt_fixedinterval_b = 1;
			break;
		case 'q':
			opt_askquit = 1;
			opt_askquit_b = 1;
			break;
		case 'Q':
			opt_askquit = 0;
			opt_askquit_b = 1;
			break;
#endif
		case 'r':
			opt_rcfile_filename = strsave(optarg);
			break;
		case 'R':
			opt_no_rcfile = 1;
			break;
		case 'c':
			copyright(); /* does not return */
		case 'V':
			version(); /* does not return */
		default: /* includes 'h' */
			usage(); /* does not return */
		}
	}
	if (argc - optind >= 2) { /* complain if more than 2 input files given */
		usage();
	}
	if (argc - optind == 0) { /* no input file given -> use stdin */
		input_filename = strsave("."); /* use . as code for stdin */
	} else {
		input_filename = strsave(argv[optind]);
	}

	/*
	 * Assign values from ~/.sel_rc (or file in opt_rcfile_filename) as defaults
	 * unless user does not want this
	 */
	if (!opt_no_rcfile) {
		FILE *rcfile;
		char *rcfilename;

		if (!opt_rcfile_filename) {
			/* Use default: ~/.sel_rc */
			char *homedir;

			homedir = getenv("HOME");
			if (!homedir) {
				homedir = "/";
			}
			/*
			 * Length of rcfilename is length of homedir +
			 * length of "/.sel_rc" (= 8) + place for NUL char (= 1)
			 */
			rcfilename = memalloc((strlen(homedir) + 8 + 1) * sizeof(char));
			strcpy(rcfilename, homedir);
			strcat(rcfilename, "/.sel_rc");
		} else if (*opt_rcfile_filename == '/') {
			/* Do not prepend current directory */
			rcfilename = strsave(opt_rcfile_filename);
		} else {
			/* Prepend current directory */
			char *curdir;

			curdir = get_cwd();
			/*
			 * Length of rcfilename is length of curdir +
			 * length of "/" (= 1) + length of opt_rcfile_filename +
			 * place for NUL char (= 1)
			 */
			rcfilename = memalloc((strlen(curdir) + 1 + strlen(opt_rcfile_filename) + 1) * sizeof(char));
			strcpy(rcfilename, curdir);
			strcat(rcfilename, "/");
			strcat(rcfilename, opt_rcfile_filename);
			memfree(curdir);
		}
		if ((rcfile = fopen(rcfilename, "r")) && !ferror(rcfile)) {
			rcin = rcfile;
			rclineno = 1;
#if YYDEBUG != 0
			rcdebug = 1;
#endif
#ifdef DEBUG
			fprintf(stderr, "rcparse: start\n");
#endif
			if (rcparse()) {
				exit(1);
			}
#ifdef DEBUG
			fprintf(stderr, "rcparse: end\n");
#endif
			fclose(rcfile);
		} else if (opt_rcfile_filename) {
			selwarning("cannot open %s", opt_rcfile_filename);
		}
		memfree(rcfilename);
	}
	rclineno = 0;

	/* Assign possibly changed opt_* variables to global option variables again */
#ifndef OUTPUT_GUI
	output_filename = opt_output_filename;
#endif
#ifdef OUTPUT_GRAPH
	if (opt_mintime_b) {
		mintime = opt_mintime;
	}
#endif
	if (opt_maxtime_b) {
		maxtime = opt_maxtime;
	}
#ifdef OUTPUT_NUM
	if (opt_data_verbosity_b) {
		verbosity = opt_verbosity;
		data = opt_data;
	}
#endif
#ifdef OUTPUT_GRAPH
	if (opt_zoom_b) {
		zoom = opt_zoom;
	}
	if (opt_transparent_b) {
		transparent = opt_transparent;
	}
#endif
#ifdef OUTPUT_GUI
	if (opt_zoommax_b) {
		zoommax = opt_zoommax;
	}
	if (opt_savetype_b) {
		savetype = opt_savetype;
	}
	if (opt_fixedinterval_b) {
		fixedinterval = opt_fixedinterval;
	}
	if (opt_askquit_b) {
		askquit = opt_askquit;
	}
#endif

#ifdef OUTPUT_GUI
	if (maxtime < 50) {
		maxtime = 50;
	}
	if (mintime + 50 > maxtime) {
		mintime = maxtime - 50;
	}
	switch (last_option) {
	case o_none:
	case o_L:
	case o_H:
		if (zoommax * (maxtime - mintime) > ZOOM_TIME_PRODUCT_MAX) {
			zoommax = ZOOM_TIME_PRODUCT_MAX / (maxtime - mintime);
		}
		if (zoommax > 20) {
			zoommax = 20;
		}
		if (zoom > zoommax) {
			zoom = zoommax;
		}
		break;
	case o_z:
		if (zoom > 20) {
			zoom = 20;
		}
		if (zoommax > 20) {
			zoommax = 20;
		}
		if (zoom > zoommax) {
			zoommax = zoom;
		}
		if (zoommax * (maxtime - mintime) > ZOOM_TIME_PRODUCT_MAX) {
			maxtime = mintime + ((ZOOM_TIME_PRODUCT_MAX / zoommax + 49) / 50) * 50;
		}
		break;
	case o_Z:
		if (zoommax > 20) {
			zoommax = 20;
		}
		if (zoom > zoommax) {
			zoom = zoommax;
		}
		if (zoommax * (maxtime - mintime) > ZOOM_TIME_PRODUCT_MAX) {
			maxtime = mintime + ((ZOOM_TIME_PRODUCT_MAX / zoommax + 49) / 50) * 50;
		}
		break;
	}
	/* Make sure zoommax is correct */
	if (zoommax == 0) {
		selerror("zoom limit must be larger than 0");
	}
	/* Set interval size */
	intervalsize = maxtime - mintime;
#endif

#ifdef OUTPUT_GRAPH
	/* Make sure mintime, maxtime, and possibly zoom are correct */
	if (mintime >= maxtime) {
		selerror("lower limit must be smaller than upper limit");
	}
	if (zoom == 0) {
		selerror("zoom must be larger than 0");
	}
#endif

#ifdef DEBUG
#define bool(x)	((x) ? "true" : "false")
#define ft(x)	(((x) == eps) ? "eps" : ((x) == png) ? "png" : ((x) == xpm) ? "xpm" : "unknown")
	fprintf(stderr, "# Current options:\n");
# ifdef OUTPUT_GRAPH
	fprintf(stderr, "lower-limit    = %lu\n", mintime);
# endif
	fprintf(stderr, "upper-limit    = %lu\n", maxtime);
# ifdef OUTPUT_NUM
	fprintf(stderr, "verbose        = %s\n", bool(verbosity));
	fprintf(stderr, "data           = %s\n", bool(data));
# endif
# ifdef OUTPUT_GRAPH
	fprintf(stderr, "zoom           = %lu\n", zoom);
	fprintf(stderr, "transparent    = %s\n", bool(transparent));
# endif
# ifdef OUTPUT_GUI
	fprintf(stderr, "zoom-limit     = %lu\n", zoommax);
	fprintf(stderr, "savetype       = %s\n", ft(savetype));
	fprintf(stderr, "fixed-interval = %s\n", bool(fixedinterval));
	fprintf(stderr, "ask-quit       = %s\n", bool(askquit));
# endif
#endif
}

void cleanup_options(void)
{
	if (input_filename) {
		memfree(input_filename);
	}
#ifndef OUTPUT_GUI
	if (output_filename) {
		memfree(output_filename);
	}
#endif
}
