/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* This file contains a replacement for the standard yyerror() function */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "error.h"
#include "sel.h"

void rcerror(const char *s, ...) /* yyerror */
/*
 * The string s is a format string, similar to printf.
 * The output will be: <progname>: error in line <rclineno>: s ...
 * Please note that it will ALWAYS print an extra newline.
 * Please also note that all printed output will go to stderr.
 */
{
	va_list ap;

	if (!strcmp(s, "parse error")) { /* don't display this general bison error */
		return;
	}
	va_start(ap, s);
	if (rclineno) {
		fprintf(stderr, "%s: error in ~/.sel_rc, line %d: ", progname, rclineno);
	} else {
		fprintf(stderr, "%s: error in ~/.sel_rc: ", progname);
	}
	vfprintf(stderr, s, ap);
	fprintf(stderr, "\n");
	va_end(ap);
}
