/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

%{

#include "error.h"
#ifdef OUTPUT_GUI
# include "gui.h"
#endif
#include "sel.h"

extern int rclineno;
extern int rclex(void);

%}

%union {
	unsigned long uval;
	int bval;
	filetype tval;
	char cval;
}

%token <uval> Lower_limit Upper_limit Zoom Verbose Data Transparent
%token <uval> Zoom_limit Savetype Fixed_Interval Ask_Quit
%token <uval> Window_Width Window_Height
%token <uval> Equals Number
%token <bval> Boolean
%token <tval> Filetype
%token <cval> Other

%start rcfile
%type <uval> rcfile

%%

rcfile	:	rcfile rc_line
	|	{} /* empty */
	;

rc_line	:	Lower_limit Equals Number
		{
#ifdef OUTPUT_GRAPH
			mintime = $3;
#endif
		}
	|	Upper_limit Equals Number
		{
			maxtime = $3;
		}
	|	Zoom Equals Number
		{
#ifdef OUTPUT_GRAPH
			zoom = (unsigned int) $3;
#endif
		}
	|	Verbose Equals Boolean
		{
#ifdef OUTPUT_NUM
			if ($3 && data) {
				rcerror("verbose and data cannot both be true");
			}
			verbosity = $3;
#endif
		}
	|	Data Equals Boolean
		{
#ifdef OUTPUT_NUM
			if ($3 && verbosity) {
				rcerror("verbose and data cannot both be true");
			}
			data = $3;
#endif
		}
	|	Transparent Equals Boolean
		{
#ifdef OUTPUT_GRAPH
			transparent = $3;
#endif
		}
	|	Zoom_limit Equals Number
		{
#ifdef OUTPUT_GUI
			zoommax = $3;
#endif
		}
	|	Savetype Equals Filetype
		{
#ifdef OUTPUT_GUI
			savetype = $3;
#endif
		}
	|	Fixed_Interval Equals Boolean
		{
#ifdef OUTPUT_GUI
			fixedinterval = $3;
#endif
		}
	|	Ask_Quit Equals Boolean
		{
#ifdef OUTPUT_GUI
			askquit = $3;
#endif
		}
	|	Window_Width Equals Number
		{
#ifdef OUTPUT_GUI
			window_width = $3;
#endif
		}
	|	Window_Height Equals Number
		{
#ifdef OUTPUT_GUI
			window_height = $3;
#endif
		}
	|	error Other
		{
			rcerror("syntax error before '%c', aborting", $2);
			YYABORT;
		}
	;

%%

int rcwrap(void)
{
	return 1;
}
