/*
 *  sel2*, selview: several programs to view schedule event listings
 *  Copyright (C) 1998-2005  Ferdy Hanssen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* This grammar is derived from a grammar designed by Ismael Ripoll */

%{

#include "actions.h"
#include "error.h"
#include "memalloc.h"
#include "sel.h"
#include "tasks.h"

extern int sellineno;
extern int sellex(void);

%}

%union {
	unsigned long uval;
	char *cval;
}

%token <uval> ACTI_t BODY_t CLAIM_t DEADLINE_t END_t
%token <uval> EXEC_t REDEF_t RESOURCE_t RSCL_t RSFR_t TASK_t
%token <uval> negative_t
%token <uval> posnum_t
%token <cval> string_t

%start chronogram
%type <uval> chronogram header header_line body body_line number

%%

chronogram	:	header BODY_t body
		;

header		:	header header_line
		|	{} /* empty */
		;

header_line	:	TASK_t posnum_t string_t
			{
				new_task((int) $2, $3);
				memfree($3);
			}
		|	RESOURCE_t posnum_t string_t
			{
				new_resource((int) $2, $3);
				memfree($3);
			}
		;

body		:	body body_line
		|	{} /* empty */
		;

body_line	:	ACTI_t posnum_t number number
			{
				if ($3 > $4) {
					selerror("deadline before release");
				} else {
					add_action_to_task(period, (int) $2, $3, $4);
				}
			}
		|	END_t number number
			{} /* silently ignore */
		|	EXEC_t posnum_t number number
			{
				if ($3 > $4) {
					selerror("finish before start");
				} else {
					add_action_to_task(exec, (int) $2, $3, $4);
				}
			}
		|	RESOURCE_t CLAIM_t posnum_t posnum_t number number
			{
				if ($5 > $6) {
					selerror("resource free before claim");
				} else {
					add_action_to_task(resource_claim, (int) $3, (int) $4,
					                                   $5, $6);
				}
			}
		|	RSCL_t posnum_t posnum_t number
			{
				selwarning("RSCL is obsolete, ignored");
			}
		|	RSFR_t posnum_t posnum_t number
			{
				selwarning("RSFR is obsolete, ignored");
			}
		|	REDEF_t DEADLINE_t posnum_t number number
			{
				add_action_to_task(deadline_redef, (int) $3, $4, $5);
			}
		;

number		:	posnum_t
			{
				$$ = $1;
			}
		|	negative_t posnum_t
			{
				/* Transform negative value into lowest possible value: 0 */
				$$ = 0;
			}
		;

%%

int selwrap(void)
{
	return 1;
}
